@@ -6,6 +6,12 @@
 #include "ee.h"
 
 #include <errno.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <fcntl.h>
+#include <unistd.h>
 
 static gint        set_root_and_quit = 0;
 static gchar       *current_dir;
@@ -15,13 +21,11 @@ static gint         start_img_list_num = 0;
 extern gint toolbar_pos;
 
 struct poptOption options[] = {
-	{"root", 'r', POPT_ARG_NONE, &set_root_and_quit, 0,
+	{"root", 'r', POPT_ARG_NONE, &set_root_and_quit, 1,
 	 N_("Set root window image and leave"), NULL},
 	{NULL, '\0', 0, NULL, 0}
 };
 
-static GtkWidget   *drag = NULL, *drag_ok = NULL;
-
 enum {
   TARGET_URI_LIST,
   TARGET_URL
@@ -53,14 +57,14 @@ typedef struct {
 } ChildData;
 
 /* Called when the child quits */
-static void
-input_func(gpointer data, gint source, GdkInputCondition condition)
+static gboolean
+input_func(GIOChannel *source, GIOCondition condition, gpointer data)
 {
   char buf[2];
   ChildData *child_data = data;
   int status;
 
-  if (read (source, buf, 2) != 2 ||
+  if (read (g_io_channel_unix_get_fd(source), buf, 2) != 2 ||
       (buf[0] != 'O') || (buf[1] != 'K')) /* failure */
     {
       g_warning ("Download failed!\n");
@@ -70,12 +74,14 @@ input_func(gpointer data, gint source, GdkInputCondition condition)
       load_file (child_data->name);
     }
 
-  waitpid (child_data->pid, &status, NULL);
+  waitpid (child_data->pid, &status, 0);
 
   g_free (child_data->name);
   g_free (child_data);
   
-  close(source);
+  close(g_io_channel_unix_get_fd(source));
+
+  return FALSE;
 }
 
 static char *download_dir = NULL;
@@ -142,6 +148,7 @@ grab_url(gchar *name)
 
       char *tail;
       ChildData *child_data;
+      GIOChannel *ioc;
 
       close(fds[1]);
 
@@ -151,11 +158,13 @@ grab_url(gchar *name)
       else
 	tail = name;
       
-      child_data = g_new (ChildData, 1);
+      child_data = g_new0 (ChildData, 1);
       child_data->name = g_strconcat (download_dir,"/",tail,NULL);
       child_data->pid = pid;
 
-      gdk_input_add (fds[0], GDK_INPUT_READ, input_func, child_data);
+      ioc = g_io_channel_unix_new (fds[0]);
+      g_io_add_watch(ioc, G_IO_IN|G_IO_HUP|G_IO_NVAL, input_func, child_data);
+      g_io_channel_unref(ioc);
     }
   else
     g_warning ("Could not fork\n");
@@ -201,7 +210,7 @@ filenames_dropped(GtkWidget * widget,
 		break;
 		
 	case TARGET_URL:
-		process_one_filename (selection_data->data);
+		process_one_filename ((char *)selection_data->data);
 		break;
 	}
 }
@@ -244,7 +253,7 @@ drag_data_get (GtkWidget        *widget,
       uri_list = g_strconcat ("file:", file, NULL);
       gtk_selection_data_set (selection_data,
 			      selection_data->target, 8,
-			      uri_list, strlen(uri_list));
+			      (void *)uri_list, strlen((char *)uri_list));
       g_free (uri_list);
     }
   else
@@ -278,7 +287,7 @@ main(int argc, char **argv)
 {
   gint i;
   GdkImlibImage *im;
-  GtkWidget *a;
+  GtkWidget *view;
   poptContext ctx;
   char **leftovers;
   
@@ -316,7 +325,7 @@ main(int argc, char **argv)
   ee_conf_load();
   
   if (start_img_list_num < 1)
-    toolbar_pos;
+    toolbar_pos++;
   
   ee_thumb_init_dirs();
   main_menu = ee_menu_new();
@@ -352,14 +361,10 @@ main(int argc, char **argv)
   ee_conf_set_options();
 
   configure_drop_on_widget(image_list);
-  if (a)
-    {
-      GtkWidget *view;
-      
-      configure_drop_on_widget(image_display);
-      view = gtk_object_get_data (GTK_OBJECT (image_display), "view");
-      configure_drag_on_widget(view);
-    }
+  configure_drop_on_widget(image_display);
+  view = gtk_object_get_data (GTK_OBJECT (image_display), "view");
+  configure_drag_on_widget(view);
+
   gtk_main();
   return 0;
 }
